<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE includes [
  <!ENTITY classScopeVars SYSTEM "classTemplateVars.inc">
  <!ENTITY propertyScopeVars SYSTEM "propertyScopeVars.inc">
  <!ENTITY getParams SYSTEM "getParams.xml">
  <!ENTITY licenseBlock SYSTEM "license.inc">
  ]>
<!-- 
    /*
    * Copyright 2004-2005 Emmanouil Batsis
    * 
    * Licensed under the GNU General Public License, Version 2.0 (the "License");
    * you may not use this file except in compliance with the License.
    * You may obtain a copy of the License at
    * 
    *      http://www.gnu.org/licenses/gpl.txt
    * 
    * Unless required by applicable law or agreed to in writing, software
    * distributed under the License is distributed on an "AS IS" BASIS,
    * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    * See the License for the specific language governing permissions and
    * limitations under the License.
    * 
    */
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
    xmlns:hbm2all="http://geekologue.com/ns/hbm4all/" xmlns:md4j="http://geekologue.com/md4j/">
    <xsl:output method="text" encoding="UTF-8" media-type="text/x-java-source"/>
    
    <xsl:include href="md4j-util.xsl"/>
    <xsl:param name="singletonDaos" select="&apos;false&apos;"/>
    
    <xsl:template name="class" match="class|joined-subclass|subclass">
        &classScopeVars;
        <xsl:value-of select="concat('package ', $package, '.dao;&#10;')"/>
        <xsl:text>&licenseBlock;</xsl:text>
        <xsl:text>&#10;</xsl:text>
        <xsl:text>import java.io.Serializable;&#10;</xsl:text>
        <xsl:text>&#10;</xsl:text>
        <xsl:text>import java.util.Map;&#10;</xsl:text>
        <xsl:text>import java.util.HashMap;&#10;</xsl:text>
        <xsl:text>import java.util.Set;&#10;</xsl:text>
        <xsl:text>import java.util.HashSet;&#10;</xsl:text>
        <xsl:text>import java.util.Date;&#10;</xsl:text>
        <xsl:text>&#10;</xsl:text>
        <xsl:text>import org.apache.commons.lang.BooleanUtils;&#10;</xsl:text>
        <xsl:text>&#10;</xsl:text>
        <xsl:text>import org.hibernate.Criteria;&#10;</xsl:text> 
        <xsl:text>import org.hibernate.Session;&#10;</xsl:text> 
        <xsl:text>import org.hibernate.criterion.Order;&#10;</xsl:text> 
        <xsl:text>import org.hibernate.criterion.Restrictions;&#10;</xsl:text> 
        <xsl:text>&#10;</xsl:text>
        <xsl:text>&#10;</xsl:text>
        <xsl:text>import org.apache.log4j.Logger;&#10;</xsl:text>
        <xsl:text>import org.apache.commons.lang.math.NumberUtils;&#10;</xsl:text>
        <xsl:text>&#10;</xsl:text>        
        <xsl:text>import gr.abiss.md4j.dao.Page;&#10;</xsl:text>
        <xsl:text>import gr.abiss.md4j.dao.hibernate.Helper;&#10;</xsl:text>
        <xsl:text>import gr.abiss.md4j.dao.DataAccessException;&#10;</xsl:text>
        <xsl:text>import gr.abiss.md4j.util.Md4jGlobals;&#10;</xsl:text>
        <xsl:text>import gr.abiss.md4j.util.ConvertUtils;&#10;</xsl:text>
        <xsl:text>&#10;</xsl:text>
        <xsl:if test="local-name() = 'class'">
            <xsl:text>import gr.abiss.md4j.dao.hibernate.AbstractHbmDAO;&#10;</xsl:text>
            <xsl:text>&#10;</xsl:text>
        </xsl:if>
        <xsl:value-of select="concat('import ', $full.classname, ';&#10;')"/>
        <xsl:call-template name="hbm2all:resolve.pojo.imports" />
        <xsl:text>&#10;</xsl:text>
        
        <xsl:text>/**&#10;</xsl:text>
        <xsl:value-of select="concat(' * Data access operations for  ', $classname, ' entities &#10;')"/>
        <xsl:text> */&#10;</xsl:text>
        <xsl:value-of select="concat('public class ', $classname, 'DAO extends ')"/>
        <xsl:choose>
            <xsl:when test="contains(local-name(), 'subclass')">
                <xsl:value-of select="concat(@extends, 'DAO {&#10;')"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="'AbstractHbmDAO {&#10;'"/> 
            </xsl:otherwise>
        </xsl:choose>
        <xsl:text>&#10;</xsl:text>
        <xsl:value-of select="concat('    private static Logger log = Logger.getLogger(', $classname, 'DAO.class);&#10;')"/>
        <xsl:text>&#10;</xsl:text>
        <xsl:text>    /**&#10;</xsl:text>
        <xsl:text>     * The property names bound by uniqueness constraints&#10;</xsl:text>
        <xsl:text>     */&#10;</xsl:text>
        <xsl:text>    public static final Set uniConstrainedPropertyNames = new HashSet();&#10;</xsl:text>
        <xsl:text>    static {&#10;</xsl:text>
        <xsl:call-template name="addUniquePropertyNamesToSet">
            <xsl:with-param name="setName" select="'uniConstrainedPropertyNames'" />
            <xsl:with-param name="spaces" select="'        '" />
        </xsl:call-template>
        <xsl:text>    }&#10;</xsl:text>
        
        <xsl:text>&#10;</xsl:text>
        <xsl:if test="$singletonDaos = 'true' and not(contains(local-name(), 'subclass'))">
            <xsl:text>&#10;</xsl:text>
            <xsl:text>    /**&#10;</xsl:text>
            <xsl:text>     * The Singleton Instance of this DAO &#10;</xsl:text>
            <xsl:text>     */&#10;</xsl:text>
            <!--xsl:value-of select="concat('    private ', $classname, 'DAO instance = null;&#10;')"/-->
            <xsl:value-of select="concat('','    protected static Object instance = null;&#10;')"/>
        </xsl:if>
        
        <xsl:variable name="constructorAcess">
            <xsl:choose>
                <xsl:when test="$singletonDaos = 'true'">
                    <xsl:text>private</xsl:text>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:text>public</xsl:text>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable><xsl:text>&#10;</xsl:text>
        <xsl:text>    /**&#10;</xsl:text>
        <xsl:text>     * Default Constructor &#10;</xsl:text>  
        <xsl:text>     */&#10;</xsl:text>
        <xsl:value-of select="concat('    ', $constructorAcess, ' ', $classname, 'DAO() {&#10;')"/>
        <xsl:value-of select="concat('        super(', $classname, '.class, &quot;', $identifier.name, '&quot;);&#10;')"/>
        <xsl:text>    }&#10;</xsl:text>
        <xsl:text>&#10;</xsl:text>
        <!-- TODO: this is not really needed. should move up to an MD4J DAO impl --> 
        <xsl:text>    /**&#10;</xsl:text>
        <xsl:text>     * Constructor to be used by subclasses essentially bubbles up the object hierarchy &#10;</xsl:text>
        <xsl:text>     * up to the constructor of AbstractHbmDAO passing along the &#10;</xsl:text>
        <xsl:text>     * Class and Identifier namme for the Entity/POJO this DAO is about &#10;</xsl:text>  
        <xsl:text>     * @see gr.abiss.md4j.hbm.AbstractHbmDAO#AbstractHbmDAO(java.lang.Class, java.io.Serializable) &#10;</xsl:text>
        <xsl:text>     */&#10;</xsl:text>
        <xsl:value-of select="concat('    protected ', $classname, 'DAO(Class clazz, String identifier) {&#10;')"/>
        <xsl:value-of select="concat('        super(', 'clazz, identifier);&#10;')"/>
        <xsl:text>    }&#10;</xsl:text>
        <xsl:if test="$singletonDaos = 'true'">
            <xsl:text>&#10;</xsl:text>
            <xsl:text>    /**&#10;</xsl:text>
            <xsl:text>     * Get the Singleton Instance of this DAO &#10;</xsl:text>
            <xsl:text>     */&#10;</xsl:text>
            <!--xsl:value-of select="concat('    public ', $classname, 'DAO getInstance(){&#10;')"/-->
            <xsl:value-of select="concat('', '    public static Object getInstance(){&#10;')"/>
            <xsl:text>        if(instance == null){&#10;</xsl:text>
            <xsl:value-of select="concat('            instance = new ', $classname, 'DAO();&#10;')"/>
            <xsl:text>        }&#10;</xsl:text>
            <xsl:text>        return instance;&#10;</xsl:text>
            <xsl:text>    }&#10;</xsl:text>
        </xsl:if>
        <xsl:text>&#10;</xsl:text>
        <!-- no need to generate for subclasses -->
        <xsl:if test="local-name() = 'class'">
            <xsl:variable name="isAssigned">
                <xsl:choose>
                    <xsl:when test="id/generator/@class='assigned'">
                        <xsl:value-of select="'true'"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="'false'"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            
            <xsl:text>    /**&#10;</xsl:text>
            <xsl:value-of select="concat('     * Creates a new ', $classname, ' instance, using the map property-value &#10;') "/>
            <xsl:text>     * pairs and persists it, then returns the identifier (PK). This method is &lt;b&gt;not&lt;/b&gt; for updating &#10;</xsl:text>
            <xsl:value-of select="concat('     * @returns the identifier of the newly persisted POJO as Serializable that can be safely casted as ', $identifier.type, ' &#10;')" /> 
            <xsl:text>     */&#10;</xsl:text>
            <xsl:value-of select="concat('    public ', 'Serializable', ' save(Map map) {&#10;')"/>
            <xsl:text>        Object pojo = this.createFromParams(map);&#10;</xsl:text>
            <!--xsl:choose>
            
            <xsl:when test="id/generator/@class='assigned'">
                <xsl:value-of select="concat('        Serializable identifier = (Serializable) map.get(&quot;', $identifier.name, '&quot;);&#10;')"/>
                <xsl:text>        this.sf.getCurrentSession().save(pojo, identifier);&#10;</xsl:text>
            </xsl:when>
            <xsl:otherwise-->
            <xsl:text>        return this.sf.getCurrentSession().save(pojo);&#10;</xsl:text>
            <!--/xsl:otherwise>
            </xsl:choose-->
            <xsl:text>    }&#10;</xsl:text>
            <xsl:text>&#10;</xsl:text>
            <xsl:text>    /**&#10;</xsl:text>
            <xsl:value-of select="concat('     * Persists the given ', $classname, ' instance  &#10;') "/>
            <xsl:text>     * This method is &lt;b&gt;not&lt;/b&gt; for updating &#10;</xsl:text>
            <xsl:value-of select="concat('     * @returns the identifier of the newly persisted POJO as Serializable that can be safely casted as ', $identifier.type, ' &#10;')" /> 
            <xsl:text>     */&#10;</xsl:text>
            
            <xsl:value-of select="concat('    public ', 'Serializable', ' save(', $classname, ' pojo) {&#10;')"/>
            <!--xsl:choose>
                <xsl:when test="$isAssigned = 'true'">
                    <xsl:text>// The ID type is assigned, get it to use it as the second param in session.save() and set it to null in the pojo &#10;</xsl:text>
                    <xsl:value-of select="concat('        Serializable identifier = (Serializable) pojo.', $identifier.getter, '();&#10;')"/>
                    <xsl:value-of select="concat('        pojo.', $identifier.setter, '(null);&#10;')"/>
                    <xsl:text>        this.sf.getCurrentSession().save(pojo, identifier);&#10;</xsl:text>
                </xsl:when>
                <xsl:otherwise-->
                    <xsl:text>        return this.sf.getCurrentSession().save(pojo);&#10;</xsl:text>
                <!--/xsl:otherwise>
            </xsl:choose-->
            <xsl:text>    }&#10;</xsl:text>
            <xsl:for-each select="id">
                &propertyScopeVars;
                <xsl:if test="not($javatype = 'String')">
                    <xsl:text>&#10;</xsl:text>
                    <xsl:text>    /**&#10;</xsl:text>
                    <xsl:text>     * Convert the identifier to the right class and call the generic load method&#10;</xsl:text>
                    <xsl:text>     * @see gr.abiss.md4j.dao.hibernate.AbstractHbmDAO#load(java.io.Serializable)&#10;</xsl:text>
                    <xsl:text>     */&#10;</xsl:text>
                    <xsl:text>    public Object load(Serializable identifier) {&#10;</xsl:text>
                    <xsl:call-template name="fromStringSerializableToTypedObject">
                        <xsl:with-param name="serializable.string.name" select="'identifier'" />
                        <xsl:with-param name="object.name" select="'typedIdentifier'" />
                    </xsl:call-template>
                    <xsl:text>        return super.load(typedIdentifier);&#10;</xsl:text>
                    <xsl:text>    }&#10;</xsl:text>
                    <xsl:text>&#10;</xsl:text>
                    <xsl:text>    /**&#10;</xsl:text>
                    <xsl:text>     * Convert the identifier to the right class and call the generic get method&#10;</xsl:text>
                    <xsl:text>     * @see gr.abiss.md4j.dao.hibernate.AbstractHbmDAO#get(java.io.Serializable)&#10;</xsl:text>
                    <xsl:text>     */&#10;</xsl:text>
                    <xsl:text>    public Object get(Serializable identifier) {&#10;</xsl:text>
                    <xsl:call-template name="fromStringSerializableToTypedObject">
                        <xsl:with-param name="serializable.string.name" select="'identifier'" />
                        <xsl:with-param name="object.name" select="'typedIdentifier'" />
                    </xsl:call-template>
                    <xsl:text>        return super.get(typedIdentifier);&#10;</xsl:text>
                    <xsl:text>    }&#10;</xsl:text>
                    <xsl:text>&#10;</xsl:text>
                    <xsl:text>    /**&#10;</xsl:text>
                    <xsl:text>     * Convert the identifier to the right class and call the generic get method&#10;</xsl:text>
                    <xsl:text>     * @see gr.abiss.md4j.dao.hibernate.AbstractHbmDAO#get(Serializable, Set)&#10;</xsl:text>
                    <xsl:text>     */&#10;</xsl:text>
                    <xsl:text>    public Map get(Serializable identifier, Set projectionProperties) {&#10;</xsl:text>
                    <xsl:call-template name="fromStringSerializableToTypedObject">
                        <xsl:with-param name="serializable.string.name" select="'identifier'" />
                        <xsl:with-param name="object.name" select="'typedIdentifier'" />
                    </xsl:call-template>
                    <xsl:text>        return super.get(typedIdentifier, projectionProperties);&#10;</xsl:text>
                    <xsl:text>    }&#10;</xsl:text>
                </xsl:if>
            </xsl:for-each>
            
        </xsl:if>
        
        <xsl:text>&#10;</xsl:text>
        <xsl:text>    /**&#10;</xsl:text>
        <xsl:value-of select="concat('     * Copy properties from the given map to the given ', $classname, ' instance &#10;')" />
        <xsl:text>     * @param map The Map object to copy properties from &#10;</xsl:text>
        <xsl:value-of select="concat('     * @tparam obj The instance of ', $classname, ' you wish to copy the properties to&#10;')" />
        <xsl:value-of select="concat('     * @throws ClassCastException in case the given object is not an instance of ', $classname, ' &#10;')" />
        <xsl:text>     * @see gr.abiss.md4j.hbm.AbstractHbmDAO#copyProperties(java.util.Map, java.lang.Object)&#10;</xsl:text>
        <xsl:text>     */&#10;</xsl:text>
        <xsl:value-of select="concat('    public void copyProperties(Map map, ', 'Object ', ' obj)  {&#10;')" />
        <xsl:value-of select="concat('        ', $classname, ' pojo = (', $classname, ') obj;&#10;')"/>
        <xsl:if test="local-name() != 'class'">
            <xsl:text>        super.copyProperties(map, pojo);&#10;</xsl:text>
        </xsl:if>
        <xsl:call-template name="pojoFromParamsMap">
            <xsl:with-param name="action" select="$actionType" />
        </xsl:call-template>
        <xsl:text>    }&#10;</xsl:text>
        <xsl:text>&#10;</xsl:text>
        
        <xsl:text>&#10;</xsl:text>
        <xsl:text>    /**&#10;</xsl:text>
        <xsl:value-of select="concat('     * Create the criteria for a ', $classname, ' search using the given &lt;code&gt;params&lt;/code&gt;, then bubble the  &#10;')"/>
        <xsl:text>     * operation to the superclass up to AbstractHbmDAO at witch point all &lt;code&gt;params&lt;/code&gt;   &#10;</xsl:text>
        <xsl:text>     * have bheen converted to criteria and the actual Criteria query can  be executed to return a Page of results. &#10;</xsl:text>
        <xsl:text>     * @see gr.abiss.md4j.hbm.AbstractHbmDAO#getPage(org.hibernate.Session, java.util.Set, java.util.Map, org.hibernate.Criteria, int, int)&#10;</xsl:text>
        <xsl:text>     * @param sess The Hibernate Session object to use for the search&#10;</xsl:text>
        <xsl:text>     * @param projectionProps The set of properties to return for each result&#10;</xsl:text>
        <xsl:text>     * @param params The set of search criteria to use &#10;</xsl:text>
        <xsl:text>     * @param criteria The criteria to use for the search, produced by each DAO in the object hierarchy using the &lt;code&gt;params&lt;/code&gt; map&#10;</xsl:text>
        <xsl:text>     * @param pageNumber The page number to return&#10;</xsl:text>
        <xsl:text>     * @param pageSize The page size&#10;</xsl:text>
        <xsl:text>     * @return The resulting Page of results&#10;</xsl:text>
        <xsl:text>     */&#10;</xsl:text>
        <xsl:text>    protected void populateCriteria(Set projectionProps, Map map, Criteria criteria) {&#10;</xsl:text>
        <xsl:if test="not(local-name() = 'class')">
            <xsl:text>        super.populateCriteria(projectionProps, map, criteria);&#10;</xsl:text>
        </xsl:if>
        <xsl:call-template name="criteriaFromParamsMap" >
            <xsl:with-param name="action" select="$actionType" />
        </xsl:call-template>
        <xsl:text>    }&#10;</xsl:text>
        <xsl:text>&#10;</xsl:text>
        
        <xsl:text>&#10;</xsl:text>
        <xsl:text>    /**&#10;</xsl:text>
        <xsl:text>     * @return The resulting Page of results&#10;</xsl:text>
        <xsl:text>     */&#10;</xsl:text>
        <xsl:text>    public void addParentOptions(Map map) {&#10;</xsl:text>
        <xsl:if test="contains(local-name(), 'subclass')">
            <xsl:text>        super.addParentOptions(map);&#10;</xsl:text>
        </xsl:if>
        <xsl:text>        Session session = this.sf.getCurrentSession();&#10;</xsl:text>
        <xsl:for-each select="many-to-one">
            <xsl:call-template name="addParentOptions" />
        </xsl:for-each>
        <xsl:text>    }&#10;</xsl:text>
            
        
        <!-- finders -->
        <xsl:for-each select="property[meta[@attribute = 'md4j:generate-finder' and . = 'true']] | many-to-one[meta[@attribute = 'md4j:generate-finder' and . = 'true']]">
            &propertyScopeVars;
            <xsl:variable name="returnType">
                <xsl:choose>
                    <xsl:when test="@unique = 'true'">
                        <xsl:value-of select="$classname"/> 
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:text>Page</xsl:text>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <!--xsl:variable name="queryMethod">
                <xsl:choose>
                    <xsl:when test="@unique = 'true'">
                        <xsl:text>uniqueResult</xsl:text>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:text>list</xsl:text>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:variable-->
            <xsl:variable name="pageParams">
                <xsl:if test="$returnType = 'Page'">
                    <xsl:text>, int pageNumber, int pageSize</xsl:text>
                </xsl:if>
            </xsl:variable>
            <xsl:variable name="finderName">
                <xsl:call-template name="hbm2all:string.toUpperCamel">
                    <xsl:with-param name="string" select="$name" />
                </xsl:call-template>
            </xsl:variable>
            <xsl:value-of select="concat('    public ', $returnType, ' findBy', $finderName, '(Serializable input', $pageParams, ') {&#10;')"/>
            <xsl:text>        Session session = this.sf.getCurrentSession();&#10;</xsl:text>
            <xsl:text>        Criteria criteria = session.createCriteria(this.daoClass);&#10;</xsl:text>
            <xsl:choose>
                <xsl:when test="local-name() = 'many-to-one'">
                    <xsl:value-of select="concat('        Criteria propertyCriteria = criteria.createCriteria(&quot;', @name, '&quot;);&#10;')" />
                    <xsl:value-of select="concat('        propertyCriteria.add(Restrictions.eq(&quot;', $identifier.name, '&quot;, input));&#10;')"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="concat('        criteria.add(Restrictions.eq(&quot;', $name, '&quot;, input));&#10;')"/>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:choose>
                <xsl:when test="$returnType = $classname">
                    <xsl:value-of select="concat('        return (', $returnType, ') criteria.uniqueResult();&#10;')"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:text>        return new HbmQueryResultPage(session, criteria, pageNumber, pageSize);&#10;</xsl:text>        
                </xsl:otherwise>
            </xsl:choose>
            <xsl:text>    }&#10;</xsl:text>
        </xsl:for-each>
        
        <!-- unique constraint check -->
        <xsl:text>    public Set getUniquePropertyNames(){ &#10;</xsl:text>
        <xsl:value-of select="concat('        return ', $classname, 'DAO.uniConstrainedPropertyNames; &#10;')"/>
        <xsl:text>    }&#10;</xsl:text>
            
        <xsl:text>&#10;}&#10;</xsl:text>
    </xsl:template>
    
    
    <xsl:template name="addUniquePropertyNamesToSet" mode="uniquePropertyNames">
        <xsl:param name="setName" />
        <xsl:param name="spaces" />
        <xsl:for-each select="property[@unique = 'true'] | id">
            <xsl:value-of select="concat($spaces, $setName, '.add(&quot;', @name, '&quot;);&#10;')"/>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template name="addParentOptions" mode="addParentOptions">
        &propertyScopeVars;
        <xsl:variable name="keyProjection">
            <xsl:call-template name="hbm2all:get.superclass.identifier">
                <xsl:with-param name="superclass" select="@class" />
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="valueProjection">
            <xsl:choose>
                <xsl:when test="meta[@attribute = 'md4j:project-property' or @attribute = 'md4j:project-property-key']">
                    <xsl:value-of select="meta[@attribute = 'md4j:project-property' or @attribute = 'md4j:project-property-key'][1]"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$keyProjection"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:value-of select="concat('        map.put(Md4jGlobals.PARENT_OPTIONS+&quot;#', 
            @name, '&quot;, Helper.getQueryResultAsMap(session.createCriteria(', 
            @class, '.class', 
            ').addOrder(Order.asc(&quot;',$valueProjection, '&quot;)), &quot;', $keyProjection, '&quot;, &quot;', $valueProjection, '&quot;));&#10;')"/>
    </xsl:template>
    
    <xsl:template name="criteriaFromParamsMap">
        <xsl:for-each select="id | property[not(@formula)] | many-to-one">
            <!--  | set | bag | list | array | map -->
            &propertyScopeVars;
            
            <xsl:value-of select="concat('           //criteriaFromParamsMap prosessing: ', local-name(), ', ', @name, ': searchType:', $searchType, ', hbmtype: ', $hbmtype, ', relatedClass: ', $relatedClass, '&#10;')"/>
            <xsl:choose>
                <!--xsl:when test="meta[@attribute = 'md4j:onsearch-ignore-for-projections'] = 'true'"-->
                    <!-- IGNORE  -->
                    <!--xsl:value-of select="concat('           // Ignoring prop: ', @name, ' of type ', $searchType, '&#10;')"/-->
                <!--/xsl:when-->
                <xsl:when test="string-length($searchType) &gt; 0">
                    <xsl:if test="local-name() = 'many-to-one' or (local-name() = 'set' or local-name() = 'bag' or local-name() = 'list' or local-name() = 'array' or local-name() = 'set')">
                        <!-- Join an association using the specified join-type, assigning an alias to the joined association. -->
                        <xsl:value-of select="concat('        criteria.createAlias(&quot;', @name, '&quot;, &quot;', @name, '&quot;, Criteria.', $join-type, ');&#10;')" />
                    </xsl:if>
                    <xsl:call-template name="fromStringMapToObject">
                        <xsl:with-param name="action" select="'Search'" />
                        <xsl:with-param name="allowNulls" select="'false'" />
                        <xsl:with-param name="nestedCode">
                            <xsl:choose>
                                <xsl:when test="local-name() = 'many-to-one' or (local-name() = 'set' or local-name() = 'bag' or local-name() = 'list' or local-name() = 'array' or local-name() = 'set')">
                                    <!-- nested criterion/alias for the relationship -->
                                    <xsl:value-of select="concat('            criteria.add(Restrictions.eq(&quot;', @name, '.', $related.identifier.name, '&quot;, ', @name, 'Id));&#10;')"/>
                                    <!--xsl:value-of select="concat('            criteria.add(Restrictions.like(&quot;', @name, $many-to-one.identifier.name, '&quot;, &quot;%&quot; +  _', @name, ' + &quot;%&quot;));&#10;')" /-->
                                </xsl:when>
                                <xsl:when test="$javatype = 'String'">
                                    <xsl:value-of select="concat('            criteria.add(Restrictions.like(&quot;', @name, '&quot;, &quot;%&quot; +  _', @name, ' + &quot;%&quot;));&#10;')" />
                                </xsl:when>
                                <xsl:when test="$javatype = 'FormFile'">                            
                                </xsl:when>
                                <xsl:when test="$javatype = 'Date'">  
                                    <xsl:value-of select="concat('            criteria.add(Expression.between(&quot;', @name, '&quot;, ConvertUtils.getStartOfDay(_', @name, '), ConvertUtils.getEndOfDay(_', @name, ')));&#10;')" />                          
                                </xsl:when>
                                <xsl:when test="$javatype = 'BigDecimal' or $javatype = 'BigInteger' or $javatype = 'Double' or $javatype = 'Float' or $javatype = 'Integer' or $javatype = 'Long' or $javatype = 'Short'  or $javatype = 'Boolean'">
                                    
                                    <xsl:value-of select="concat('            criteria.add(Restrictions.eq(&quot;', @name, '&quot;, _', @name, '));&#10;')" />
                                    <!--/xsl:otherwise>
                                        </xsl:choose-->
                                </xsl:when>
                                
                            </xsl:choose>
                        </xsl:with-param>
                        <xsl:with-param name="resolveRelations" select="'true'" />
                    </xsl:call-template>
                </xsl:when>
            </xsl:choose>
        </xsl:for-each>
    </xsl:template>
</xsl:stylesheet>