<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE includes [
  <!ENTITY classScopeVars SYSTEM "classTemplateVars.inc">
  <!ENTITY propertyScopeVars SYSTEM "propertyScopeVars.inc">
  <!ENTITY getEJB SYSTEM "getEJB.inc">
  <!ENTITY getParams SYSTEM "getParams.xml">
  ]>
<!-- 
    /*
    * Copyright 2004-2005 Emmanouil Batsis
    * 
    * Licensed under the GNU General Public License, Version 2.0 (the "License");
    * you may not use this file except in compliance with the License.
    * You may obtain a copy of the License at
    * 
    *      http://www.gnu.org/licenses/gpl.txt
    * 
    * Unless required by applicable law or agreed to in writing, software
    * distributed under the License is distributed on an "AS IS" BASIS,
    * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    * See the License for the specific language governing permissions and
    * limitations under the License.
    * 
    */
-->
<xsl:transform xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
    xmlns:hbm2all="http://geekologue.com/ns/hbm4all/" xmlns:md4j="http://geekologue.com/md4j/">
    <xsl:param name="package" select="'gr.geekologue.hbm2all.web.default'"/>
    <xsl:variable name="hbmGlobalPackage" select="/hibernate-mapping/@package"/>
    <xsl:param name="pageAttributeName" select="'resultsPage'"/>
    <xsl:param name="usePropertyNamesAsI18nKeys" select="'true'"/>
    <xsl:param name="actionType" select="''"/>
    
    <xsl:param name="ejbViewMethod" select="'both'"/>
    <xsl:param name="remoteEjbViewMethod" select="'false'"/>
    <!-- TODO: make templates to add legal info etc in generated files -->
    <xsl:param name="fileInfo" select="''"/>
    <xsl:variable name="uppercase" select="'ABCDEFGHIJKLMNOPQRSTUVWXYZ'" />
    <xsl:variable name="lowercase" select="'abcdefghijklmnopqrstuvwxyz'" />
    <xsl:variable name="actionLowcase">
        <xsl:call-template name="string.toLowCase">
            <xsl:with-param name="string" select="$actionType" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="validate">
        <xsl:choose>
            <xsl:when test="$actionType = 'Create' or $actionType = 'Update' or $actionType = 'Search'">
                <xsl:value-of  select="'true'"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of  select="'false'"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    <xsl:template name="string.toLowCase">
        <xsl:param name="string" />
        <xsl:value-of select="translate($string, $uppercase, $lowercase)"/>
    </xsl:template>
    <xsl:template match="/">
        <xsl:apply-templates/>
    </xsl:template>
    <xsl:template match="hibernate-mapping">
        <xsl:apply-templates select="class|joined-subclass | subclass"/>
    </xsl:template>
    
    <xsl:template name="hbm2all:name.toUnqualifiedClassname">
        <xsl:param name="name"/>
        <xsl:choose>
            <xsl:when test="contains($name, '.')">
                <xsl:call-template name="hbm2all:name.toUnqualifiedClassname">
                    <xsl:with-param name="name" select="substring-after($name, '.')"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$name"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template name="propertyName.toGetter">
        <xsl:param name="propertyName"/>
        <xsl:text>get</xsl:text>
        <xsl:call-template name="hbm2all:string.toUpperCamel">
            <xsl:with-param name="string" select="$propertyName"/>
        </xsl:call-template>
    </xsl:template>
    <xsl:template name="propertyName.toSetter">
        <xsl:param name="propertyName"/>
        <xsl:text>set</xsl:text>
        <xsl:call-template name="hbm2all:string.toUpperCamel">
            <xsl:with-param name="string" select="$propertyName"/>
        </xsl:call-template>
    </xsl:template>
    <!-- TODO: rename, this is not only used for pojos anymore -->
    <xsl:template name="hbm2all:resolve.pojo.imports">
        <xsl:param name="extends" />
        <xsl:choose>
            <xsl:when test="$extends = 'Action'">
                <xsl:text>import org.apache.struts.action.Action;&#10;</xsl:text>
            </xsl:when>
            <xsl:when test="$extends = 'DispatchAction'">
                <xsl:text>import org.apache.struts.actions.DispatchAction;&#10;</xsl:text>
            </xsl:when>
            <xsl:when test="$extends = 'ValidatorForm'">
                <xsl:text>import org.apache.struts.validator.ValidatorForm;&#10;</xsl:text>
            </xsl:when>
        </xsl:choose>
        <xsl:for-each select="many-to-one">
            <xsl:variable name="unqualifiedClassname">
                <xsl:call-template name="hbm2all:name.toUnqualifiedClassname">
                    <xsl:with-param name="name" select="@class" />
                </xsl:call-template>
            </xsl:variable>
            <xsl:value-of select="concat('import ', $package, '.', $unqualifiedClassname, ';&#10;')"/>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template name="hbm2all:get.superclass.identifier">
        <xsl:param name="superclass" />
        <xsl:choose>
            <xsl:when test="/hibernate-mapping/class[@name = $superclass]">
                <xsl:value-of select="/hibernate-mapping/class[@name = $superclass]/id/@name"/>
            </xsl:when>
            <xsl:when test="/hibernate-mapping/*[contains(local-name(), 'subclass')][@name = $superclass]">
                <xsl:call-template name="hbm2all:get.superclass.identifier">
                    <xsl:with-param name="superclass">
                        <xsl:call-template name="hbm2all:name.toUnqualifiedClassname">
                            <xsl:with-param name="name">
                                <xsl:value-of select="/hibernate-mapping/*[contains(local-name(), 'subclass')][@name = $superclass]/@extends"/>
                            </xsl:with-param>
                        </xsl:call-template>
                    </xsl:with-param>
                </xsl:call-template>
            </xsl:when>
            <xsl:when test="document(concat($superclass, '.hbm.xml'), .)/hibernate-mapping/class[@name = $superclass]">
                <xsl:value-of select="document(concat($superclass, '.hbm.xml'), .)/hibernate-mapping/class[@name = $superclass]/id/@name"/>
            </xsl:when>
            <xsl:when test="document(concat($superclass, '.hbm.xml'), .)/hibernate-mapping/*[contains(local-name(), 'subclass')][@name = $superclass]">
                <xsl:call-template name="hbm2all:get.superclass.identifier">
                    <xsl:with-param name="superclass">
                        <xsl:call-template name="hbm2all:name.toUnqualifiedClassname">
                            <xsl:with-param name="name">
                                <xsl:value-of select="document(concat($superclass, '.hbm.xml'), .)/hibernate-mapping/*[contains(local-name(), 'subclass')][@name = $superclass]/@extends"/>
                            </xsl:with-param>
                        </xsl:call-template>
                    </xsl:with-param>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:message terminate="yes">Could not find identifier name for <xsl:value-of select="$superclass"/> </xsl:message>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template name="hbm2all:get.superclass.identifier.type">
        <xsl:param name="superclass" />
        <xsl:choose>
            <xsl:when test="/hibernate-mapping/class[@name = $superclass]">
                <xsl:value-of select="/hibernate-mapping/class[@name = $superclass]/id/@type"/>
            </xsl:when>
            <xsl:when test="/hibernate-mapping/*[contains(local-name(), 'subclass')][@name = $superclass]">
                <xsl:call-template name="hbm2all:get.superclass.identifier.type">
                    <xsl:with-param name="superclass">
                        <xsl:call-template name="hbm2all:name.toUnqualifiedClassname">
                            <xsl:with-param name="name">
                                <xsl:value-of select="/hibernate-mapping/*[contains(local-name(), 'subclass')][@name = $superclass]/@extends"/>
                            </xsl:with-param>
                        </xsl:call-template>
                    </xsl:with-param>
                </xsl:call-template>
            </xsl:when>
            <xsl:when test="document(concat($superclass, '.hbm.xml'), .)/hibernate-mapping/class[@name = $superclass]">
                <xsl:value-of select="document(concat($superclass, '.hbm.xml'), .)/hibernate-mapping/class[@name = $superclass]/id/@type"/>
            </xsl:when>
            <xsl:when test="document(concat($superclass, '.hbm.xml'), .)/hibernate-mapping/*[contains(local-name(), 'subclass')][@name = $superclass]">
                <xsl:call-template name="hbm2all:get.superclass.identifier.type">
                    <xsl:with-param name="superclass">
                        <xsl:call-template name="hbm2all:name.toUnqualifiedClassname">
                            <xsl:with-param name="name">
                                <xsl:value-of select="document(concat($superclass, '.hbm.xml'), .)/hibernate-mapping/*[contains(local-name(), 'subclass')][@name = $superclass]/@extends"/>
                            </xsl:with-param>
                        </xsl:call-template>
                    </xsl:with-param>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:message terminate="yes">Could not find superclass identifier type!</xsl:message>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template name="hbm2all:get.superclass.identifier.generator">
        <xsl:param name="superclass" />
        <xsl:choose>
            <xsl:when test="/hibernate-mapping/class[@name = $superclass]">
                <xsl:value-of select="/hibernate-mapping/class[@name = $superclass]/id/generator/@class"/>
            </xsl:when>
            <xsl:when test="/hibernate-mapping/*[contains(local-name(), 'subclass')][@name = $superclass]">
                <xsl:call-template name="hbm2all:get.superclass.identifier.generator">
                    <xsl:with-param name="superclass">
                        <xsl:call-template name="hbm2all:name.toUnqualifiedClassname">
                            <xsl:with-param name="name">
                                <xsl:value-of select="/hibernate-mapping/*[contains(local-name(), 'subclass')][@name = $superclass]/@extends"/>
                            </xsl:with-param>
                        </xsl:call-template>
                    </xsl:with-param>
                </xsl:call-template>
            </xsl:when>
            <xsl:when test="document(concat($superclass, '.hbm.xml'), .)/hibernate-mapping/class[@name = $superclass]">
                <xsl:value-of select="document(concat($superclass, '.hbm.xml'), .)/hibernate-mapping/class[@name = $superclass]/id/generator/@class"/>
            </xsl:when>
            <xsl:when test="document(concat($superclass, '.hbm.xml'), .)/hibernate-mapping/*[contains(local-name(), 'subclass')][@name = $superclass]">
                <xsl:call-template name="hbm2all:get.superclass.identifier.generator">
                    <xsl:with-param name="superclass">
                        <xsl:call-template name="hbm2all:name.toUnqualifiedClassname">
                            <xsl:with-param name="name">
                                <xsl:value-of select="document(concat($superclass, '.hbm.xml'), .)/hibernate-mapping/*[contains(local-name(), 'subclass')][@name = $superclass]/@extends"/>
                            </xsl:with-param>
                        </xsl:call-template>
                    </xsl:with-param>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:message terminate="yes">Could not find superclass identifier generator!</xsl:message>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    
    <xsl:template name="hbm2all:hbmtype.toActionFormType">
        <xsl:param name="hbmtype" />
        <xsl:choose>
            <xsl:when test="$hbmtype = 'binary' or $hbmtype = 'clob' or $hbmtype = 'blob' or string(meta[@attribute = 'md4j:is-file'][1]) = 'true'">
                <xsl:text>FormFile</xsl:text>
            </xsl:when>
            <!--TODO: metadata to distinguish between textarea and file -->
            <xsl:otherwise>
                <xsl:text>String</xsl:text>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template name="hbm2all:hbmtype.toSearchType">
        <xsl:param name="hbmtype" />
        <xsl:choose>
            <xsl:when test="$hbmtype = 'string' or $hbmtype = 'text' or $hbmtype = 'java.lang.String'">
                <xsl:text>String</xsl:text>
            </xsl:when>
            <xsl:when test="$hbmtype = 'integer' or $hbmtype = 'int' or $hbmtype = 'java.lang.Integer'">
                <xsl:text>Integer</xsl:text>
            </xsl:when>
            <xsl:when test="$hbmtype = 'float' or $hbmtype = 'java.lang.Float'">
                <xsl:text>Float</xsl:text>
            </xsl:when>
            <xsl:when test="$hbmtype = 'short' or $hbmtype = 'java.lang.Short'">
                <xsl:text>Short</xsl:text>
            </xsl:when>
            <xsl:when test="$hbmtype = 'long' or $hbmtype = 'java.lang.Long'">
                <xsl:text>Long</xsl:text>
            </xsl:when>
            <xsl:when test="$hbmtype = 'double' or $hbmtype = 'java.lang.Double'">
                <xsl:text>Double</xsl:text>
            </xsl:when>
            <xsl:when test="$hbmtype = 'java.math.BigDecimal' or $hbmtype = 'big_decimal'">
                <xsl:text>java.math.BigDecimal</xsl:text>
            </xsl:when>
            <xsl:when test="$hbmtype = 'java.math.BigInteger' or $hbmtype = 'big_integer'">
                <xsl:text>java.math.BigInteger</xsl:text>
            </xsl:when>
            <xsl:when test="$hbmtype = 'java.lang.Boolean' or $hbmtype = 'Boolean' or $hbmtype = 'bool' or $hbmtype = 'boolean' or $hbmtype = 'yes_no' or $hbmtype = 'true_false'">
                <xsl:text>Boolean</xsl:text>
            </xsl:when>
            <xsl:when test="$hbmtype = 'character' or $hbmtype = 'char'">
                <xsl:text>Char</xsl:text>
            </xsl:when>
            <xsl:when test="$hbmtype = 'date' or $hbmtype = 'time' or $hbmtype = 'timestamp' or $hbmtype = 'java.sql.Timestamp'">
                <xsl:text>Date</xsl:text>
            </xsl:when>
            <xsl:when test="$hbmtype = 'calendar' or $hbmtype = 'calendar_date'">
                <xsl:text></xsl:text>
            </xsl:when>
            <xsl:when test="$hbmtype = 'locale'">
                <xsl:text></xsl:text>
            </xsl:when>
            <xsl:when test="$hbmtype = 'timezone'">
                <xsl:text></xsl:text>
            </xsl:when>
            <xsl:when test="$hbmtype = 'currency'">
                <xsl:text></xsl:text>
            </xsl:when>
            <xsl:when test="$hbmtype = 'binary'">
                <xsl:text>FormFile</xsl:text>
            </xsl:when>
            <!--TODO: metadata to distinguish between textarea and file -->
            <xsl:when test="$hbmtype = 'clob'">
                <xsl:text>FormFile</xsl:text>
            </xsl:when>
            <xsl:when test="$hbmtype = 'blob'">
                <xsl:text>FormFile</xsl:text>
            </xsl:when>
        </xsl:choose>
    </xsl:template>
    <xsl:template name="hbm2all:type.hbm2JavaType">
        <xsl:param name="type" />
        <xsl:choose>
            <xsl:when test="$type = 'string' or $type='text' or $type = 'java.lang.String'">
                <xsl:text>String</xsl:text>
            </xsl:when>
            <xsl:when test="$type = 'integer' or $type = 'int' or $type = 'java.lang.Integer'">
                <xsl:text>Integer</xsl:text>
            </xsl:when>
            <xsl:when test="$type = 'float' or $type = 'java.lang.Float'">
                <xsl:text>Float</xsl:text>
            </xsl:when>
            <xsl:when test="$type = 'short' or $type = 'java.lang.Short'">
                <xsl:text>Short</xsl:text>
            </xsl:when>
            <xsl:when test="$type = 'long' or $type = 'java.lang.Long'">
                <xsl:text>Long</xsl:text>
            </xsl:when>
            <xsl:when test="$type = 'big_decimal'">
                <xsl:text>java.math.BigDecimal</xsl:text>
            </xsl:when>
            <xsl:when test="$type = 'java.lang.Boolean' or $type = 'Boolean' or $type = 'yes_no' or $type = 'true_false'">
                <xsl:text>Boolean</xsl:text>
            </xsl:when>
            <xsl:when test="$type = 'boolean'">
                <xsl:text>Boolean</xsl:text>
            </xsl:when>
            <xsl:when test="$type = 'character'">
                <xsl:text>char</xsl:text>
            </xsl:when>
            <xsl:when test="$type = 'date' or $type = 'java.util.Date'">
                <xsl:text>java.util.Date</xsl:text>
            </xsl:when>
            <xsl:when test="$type = 'time' or $type = 'timestamp' or $type = 'java.sql.Timestamp'">
                <xsl:text>java.sql.Timestamp</xsl:text>
            </xsl:when>
            <xsl:when test="$type = 'calendar' or $type = 'calendar_date'">
                <xsl:text>java.util.Calendar</xsl:text>
            </xsl:when>
            <xsl:when test="$type = 'locale'">
                <xsl:text>java.util.Locale</xsl:text>
            </xsl:when>
            <xsl:when test="$type = 'timezone'">
                <xsl:text>java.util.TimeZone</xsl:text>
            </xsl:when>
            <xsl:when test="$type = 'currency'">
                <xsl:text>java.util.Currency</xsl:text>
            </xsl:when>
            <xsl:when test="$type = 'class'">
                <xsl:text>Class</xsl:text>
            </xsl:when>
            <xsl:when test="$type = 'serializable'">
                <xsl:text>java.io.Serializable</xsl:text>
            </xsl:when>
            <xsl:when test="$type = 'binary'">
                <xsl:text>byte[]</xsl:text>
            </xsl:when>
            <xsl:when test="$type = 'clob'">
                <xsl:text>java.sql.Clob</xsl:text>
            </xsl:when>
            <xsl:when test="$type = 'blob'">
                <xsl:text>java.sql.Blob</xsl:text>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$type" />
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template name="hbm2all:string.toLowerCamel">
        <xsl:param name="string" />
        <xsl:value-of select="translate(substring($string, 1, 1), $uppercase, $lowercase)" />
        <xsl:value-of select="substring($string, 2)" />
    </xsl:template>
    
    <xsl:template name="hbm2all:string.toUpperCamel">
        <xsl:param name="string" />
        <xsl:value-of select="translate(substring($string, 1, 1), $lowercase, $uppercase)" />
        <xsl:value-of select="substring($string, 2)" />
    </xsl:template>
    
    <xsl:template name="hbm2all:getDateFormat">
        <xsl:param name="meta-value" />
        <xsl:choose>
            <xsl:when test="starts-with($meta-value, 'DateFormat.') 
                or starts-with($meta-value, 'SimpleDateFormat.')">
                <xsl:value-of select="$meta-value"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="concat('&quot;', $meta-value, '&quot;')"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- 
        Convert strings from a map to the appropriate object.
        This template assumes that:
        * Its called while generating a DAO if told to resolve POJOs
        * the Map name is 'map'
        And produces something that usually looks like:
        String  _string{@name} = params.get({@name});
        // if target type is not a string
        if(_string{@name} != null){
            {$type} _{@name} = convertCode({$howtoget});
            {$nestedCode}  
        }
    -->
    
    <xsl:template name="fromStringIdToObject" mode="fromStringIdToObject">
        <xsl:param name="stringName" select="'defaultStringName'" />
        <xsl:param name="objectName" select="'defaultObjectName'" />
        <xsl:param name="indent" select="'            '" />
        <xsl:param name="identifier.type"></xsl:param>
        
        
        <xsl:variable name="unqualified-javatype">
            <xsl:call-template name="hbm2all:name.toUnqualifiedClassname">
                <xsl:with-param name="name" select="$identifier.type"  />
            </xsl:call-template>
        </xsl:variable>
        
        <xsl:choose>
            <xsl:when test="$unqualified-javatype = 'String'">
                <xsl:value-of select="concat($indent, 'String ', $objectName, ' = (String) ', $stringName, ';&#10;', '')"/>
            </xsl:when>
            <xsl:when test="$unqualified-javatype = 'Boolean'">
                <xsl:value-of select="concat($indent, 'Boolean ', $objectName, ' = BooleanUtils.toBooleanObject', '(', $stringName, ');&#10;')"/>
            </xsl:when>
            <xsl:when test="$unqualified-javatype = 'Date'">
                <xsl:variable name="pattern">
                    <xsl:choose>
                        <xsl:when test="meta[@attribute = 'md4j:date-format']">
                            <xsl:value-of select="meta[@attribute = 'md4j:date-format']"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:text>MM/dd/yyyy</xsl:text>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                <xsl:value-of select="concat($indent, $identifier.type, ' ', $objectName, ' = ConvertUtils.parseDate(', $stringName, ', &quot;', $pattern, '&quot;);&#10;')"/>
            </xsl:when>
            <xsl:when test="$unqualified-javatype = 'BigDecimal' or $unqualified-javatype = 'BigInteger' or $unqualified-javatype = 'Double' or $unqualified-javatype = 'Float' or $unqualified-javatype = 'Integer' or $unqualified-javatype = 'Long'">
                <xsl:value-of select="concat($indent, $identifier.type, ' ', $objectName, ' = NumberUtils.create', $unqualified-javatype, '(', $stringName, ');&#10;')"/>
            </xsl:when>
            <xsl:when test="$unqualified-javatype = 'Short'">
                <xsl:value-of select="concat($indent, $identifier.type, ' ', $objectName, ' = new Short(NumberUtils.createNumber(', $stringName, ').shortValue());&#10;')"/>
            </xsl:when>
        </xsl:choose>
    </xsl:template>
    
    
    <xsl:template name="fromStringMapToObject">
        <xsl:param name="action" />
        <xsl:param name="resolveRelations" select="'false'" />
        <xsl:param name="allowNulls" select="'true'" />
        <xsl:param name="nestedCode"/>
        
        &propertyScopeVars;
        <!--xsl:value-of select="concat('               // fromStringMapToObject prosessing: ', local-name(), ' element&#10;')"/-->
        <xsl:variable name="getFromMapCondition">
            <xsl:choose>
                <xsl:when test="$allowNulls = 'true'">
                    <xsl:value-of select="concat('        if ( map.containsKey(&quot;', $name,'&quot;)) {&#10;')"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="concat('        if ( map.get(&quot;', $name, '&quot;) != null) {&#10;')"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <!--xsl:value-of select="concat('/** javatype is ', $javatype, ', resolveRelations is ', $resolveRelations, '*/&#10;')"/-->
        <xsl:choose>
            <xsl:when test="$resolveRelations = 'true' and local-name() = 'many-to-one' or  local-name() = 'set' or  local-name() = 'list' or  local-name() = 'bag' or  local-name() = 'array' or  local-name() = 'map' ">
                
                <xsl:value-of select="concat('        Serializable', ' _', $name,'Identifier = (Serializable) map.get(&quot;', $name, '&quot;);&#10;')"/>
                <xsl:value-of select="concat('        if (_', $name,'Identifier !=  null) {&#10;')"/>
                
                <xsl:if test="$action != 'Search'">
                    <xsl:value-of select="concat('            ', $relatedClass, ' _', $name,' = null;&#10;')"/>
                </xsl:if>
                <xsl:call-template name="fromStringSerializableIdentifierOfClassToTypedObject">
                    <xsl:with-param name="serializable.string.name" select="concat(' _', $name,'Identifier')" />
                    <xsl:with-param name="object.name" select="concat($name,'Id')" />
                    <xsl:with-param name="class" select="$relatedClass" />
                    <xsl:with-param name="action" select="$action" />
                </xsl:call-template>
                <xsl:if test="$action != 'Search'">
                    <xsl:value-of select="concat('            _', $name,' = (', @class, ') this.sf.getCurrentSession().load(', $relatedClass, '.class, ', @name, 'Id);&#10;')"/>
                </xsl:if>
                <xsl:value-of select="$nestedCode"/>
                <xsl:text>        }&#10;</xsl:text>
            </xsl:when>
            <xsl:when test="$javatype = 'String'">
                <xsl:value-of select="$getFromMapCondition"/>
                
                <xsl:value-of select="concat('            String _', $name, ' = (String) map.get(&quot;', $name, '&quot;);&#10;')"/>
                <xsl:value-of select="$nestedCode"/>
                <xsl:text>        }&#10;</xsl:text>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$getFromMapCondition"/>
                
                <xsl:value-of select="concat('            String _string', $name, ' = (String) map.get(&quot;', $name, '&quot;);&#10;')"/>
                <xsl:choose>
                    <xsl:when test="$javatype = 'Boolean'">
                        <xsl:variable name="default">
                            <xsl:choose>
                                <xsl:when test="meta[@attribute = 'md4j:boolean-default']">
                                    <xsl:value-of select="meta[@attribute = 'md4j:boolean-default']"/>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:text>false</xsl:text>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:variable>
                        <!-- adddefault and  defaultIfNull to handle weird input -->
                        <xsl:value-of select="concat('            Boolean _', $name, ' = BooleanUtils.toBooleanObject', '(_string', $name, ');&#10;')"/>
                    </xsl:when>
                    <xsl:when test="$javatype = 'java.util.Date' or $javatype = 'java.sql.Timestamp'">
                        <xsl:variable name="pattern">
                            <xsl:choose>
                                <xsl:when test="meta[@attribute = 'md4j:date-format']">
                                    <xsl:value-of select="meta[@attribute = 'md4j:date-format']"/>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:text>MM/dd/yyyy</xsl:text>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:variable>
                        <xsl:if test="$javatype = 'java.util.Date'">
                            <xsl:value-of select="concat('            java.util.Date _', $name, ' = ConvertUtils.parseDate(_string', $name, ', &quot;', $pattern, '&quot;);&#10;')"/>
                        </xsl:if>
                        <xsl:if test="$javatype = 'java.sql.Timestamp'">
                            <xsl:value-of select="concat('            java.sql.Timestamp _', $name, ' = new java.sql.Timestamp(ConvertUtils.parseDate(_string', $name, ', &quot;', $pattern, '&quot;).getTime());&#10;')"/>
                        </xsl:if>
                        
                        <!-- TODO: add range end -->
                        <!--xsl:if test="meta[@attribute = concat('md4j:on', $action, '-use-range')] = 'true'">
                            
                            <xsl:value-of select="concat('        String _string', $name, ' = (String) map.get(&quot;', $name, '&quot;);&#10;')"/>
                            <xsl:value-of select="concat('        java.util.Date _', $name, 'RangeEnd = new SimpleDateFormat(&quot;', $pattern, '&quot;).parse((String) map.get(&quot;', $name, '&quot;));&#10;')"/>
                        </xsl:if-->
                    </xsl:when>
                    <!--xsl:when test="$javatype = 'FormFile'">
                        <xsl:value-of select="concat('        File ',$name, ' = new File();&#10;')"/>
                        <xsl:value-of select="concat('        FormFile ',$name, 'FormFile = ', $howtoget, ';&#10;')"/>
                        <xsl:value-of select="concat('        ',$name, '.setContentType(', $name, 'FormFile.getContentType());&#10;')"/>
                        <xsl:value-of select="concat('        ',$name, '.setFileSize(', $name, 'FormFile.getFileSize());&#10;')"/>
                        <xsl:value-of select="concat('        ',$name, '.setFileName(', $name, 'FormFile.getFileName());&#10;')"/>
                        <xsl:value-of select="concat('        ',$name, '.setInputStream(', $name, 'FormFile.getInputStream());&#10;')"/>
                        </xsl:when-->
                    <xsl:when test="$javatype = 'java.math.BigDecimal' or $javatype = 'java.math.BigInteger' or $javatype = 'Double' or $javatype = 'Float' or $javatype = 'Integer' or $javatype = 'Long'">
                        <xsl:variable name="unqualifiedJavatype">
                            <xsl:call-template name="hbm2all:name.toUnqualifiedClassname">
                                <xsl:with-param name="name" select="$javatype"/>
                            </xsl:call-template>
                        </xsl:variable>
                        <xsl:value-of select="concat('            ', $javatype, ' _', $name,' = NumberUtils.create', $unqualifiedJavatype, '(_string', $name, ');&#10;')"/>
                    </xsl:when>
                    <xsl:when test="$javatype = 'Short'">
                        <xsl:value-of select="concat('            ', $javatype, ' _', $name,' = new Short(NumberUtils.createNumber(_string', $name, ').shortValue());&#10;')"/>
                    </xsl:when>
                </xsl:choose>
                <xsl:value-of select="$nestedCode"/>
                <xsl:value-of select="'        }&#10;'"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template name="fromStringSerializableIdentifierOfClassToTypedObject">
        <xsl:param name="serializable.string.name" select="'stringSerializable'" />
        <xsl:param name="object.name" select="'typedObject'" />
        <xsl:param name="class" />
        <xsl:variable name="javatype">
            <xsl:call-template name="hbm2all:type.hbm2JavaType">
                <xsl:with-param name="type">
                    <xsl:call-template name="hbm2all:get.superclass.identifier.type">
                        <xsl:with-param name="superclass" select="$class" />
                    </xsl:call-template>
                </xsl:with-param>
            </xsl:call-template>
        </xsl:variable>
        <xsl:call-template name="serializableStringToObject">
            <xsl:with-param name="extraspace" select="'    '" />
            <xsl:with-param name="javatype" select="$javatype"/>
            <xsl:with-param name="object.name" select="$object.name"/>
            <xsl:with-param name="serializable.string.name" select="$serializable.string.name"/>
        </xsl:call-template>
    </xsl:template>
    
    <!-- from serializable (string) to specific type -->
    <xsl:template name="fromStringSerializableToTypedObject">
        <xsl:param name="serializable.string.name" select="'stringSerializable'" />
        <xsl:param name="object.name" select="'typedObject'" />
        &propertyScopeVars;
        <xsl:call-template name="serializableStringToObject">
            <xsl:with-param name="javatype" select="$javatype"/>
            <xsl:with-param name="object.name" select="$object.name"/>
            <xsl:with-param name="serializable.string.name" select="$serializable.string.name"/>
        </xsl:call-template>
        
    </xsl:template>
    <xsl:template name="serializableStringToObject">
        <xsl:param name="javatype"/>
        <xsl:param name="object.name"/>
        <xsl:param name="serializable.string.name"/>
        <xsl:param name="extraspace" select="''"/>
        
        
        <xsl:variable name="unqualified-javatype">
            <xsl:call-template name="hbm2all:name.toUnqualifiedClassname">
                <xsl:with-param name="name" select="$javatype"  />
            </xsl:call-template>
        </xsl:variable>
        <!--xsl:value-of select="concat('/** javatype is ', $unqualified-javatype, ', resolveRelations is ', $resolveRelations, '*/&#10;')"/-->
        <xsl:choose>
            <xsl:when test="$unqualified-javatype = 'String'">
                <xsl:value-of select="concat($extraspace, '        String ', $object.name, ' = (String) ', $serializable.string.name, ';&#10;')"/>
            </xsl:when>
            <xsl:when test="$unqualified-javatype = 'Boolean'">
                <xsl:variable name="default">
                    <xsl:choose>
                        <xsl:when test="meta[@attribute = 'md4j:boolean-default']">
                            <xsl:value-of select="meta[@attribute = 'md4j:boolean-default']"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:text>false</xsl:text>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                <!-- add defaultIfNull to handle weird input -->
                <xsl:value-of select="concat($extraspace, '        Boolean ', $object.name, ' = BooleanUtils.toBooleanObject', '((String) ', $serializable.string.name, ');&#10;')"/>
            </xsl:when>
            <xsl:when test="$unqualified-javatype = 'Date' or $unqualified-javatype = 'java.util.Date'">
                <xsl:variable name="pattern">
                    <xsl:choose>
                        <xsl:when test="meta[@attribute = 'md4j:date-format']">
                            <xsl:value-of select="meta[@attribute = 'md4j:date-format']"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:text>MM/dd/yyyy</xsl:text>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                <xsl:value-of select="concat($extraspace, '        java.util.Date ', $object.name, ' = new SimpleDateFormat(&quot;', $pattern, '&quot;).parse((String) ', $serializable.string.name, ');&#10;')"/>
            </xsl:when>
            <!--xsl:when test="$unqualified-javatype = 'FormFile'">
                <xsl:value-of select="concat('        File ',$object.name, ' = new File();&#10;')"/>
                <xsl:value-of select="concat('        FormFile ',$object.name, 'FormFile = ', $howtoget, ';&#10;')"/>
                <xsl:value-of select="concat('        ',$object.name, '.setContentType(', $object.name, 'FormFile.getContentType());&#10;')"/>
                <xsl:value-of select="concat('        ',$object.name, '.setFileSize(', $object.name, 'FormFile.getFileSize());&#10;')"/>
                <xsl:value-of select="concat('        ',$object.name, '.setFileName(', $object.name, 'FormFile.getFileName());&#10;')"/>
                <xsl:value-of select="concat('        ',$object.name, '.setInputStream(', $object.name, 'FormFile.getInputStream());&#10;')"/>
                </xsl:when-->
            <xsl:when test="$unqualified-javatype = 'BigDecimal' or $unqualified-javatype = 'BigInteger' or $unqualified-javatype = 'Double' or $unqualified-javatype = 'Float' or $unqualified-javatype = 'Integer' or $unqualified-javatype = 'Long' or $unqualified-javatype = 'Short'">
                <xsl:value-of select="concat($extraspace, '        ', $javatype, ' ', $object.name,' = NumberUtils.create', $unqualified-javatype, '((String) ', $serializable.string.name, ');&#10;')"/>
            </xsl:when>
            
        </xsl:choose>
        
    </xsl:template>
    
    
    <xsl:template name="pojoFromParamsMap">
        <xsl:param name="action"/>
        <xsl:for-each select="id | property[not(@formula)] | many-to-one">
            &propertyScopeVars;
            <xsl:choose>
                <xsl:when test="meta/@attribute[local-name() = concat('md4j:on', $action, '-ignore') and string(meta/@attribute[local-name() = concat('md4j:on', $action, '-ignore')]) = 'true']">
                    <!-- IGNORE  -->
                    <xsl:value-of select="concat('            /* Ignoring prop: ', @name, ' of type ', $searchType, ' */')"/>
                </xsl:when>
                <xsl:when test="string-length($searchType) &gt; 0">
                    <xsl:call-template name="fromStringMapToObject">
                        <xsl:with-param name="action" select="'create'" />
                        <xsl:with-param name="nestedCode">
                            <xsl:value-of select="concat('            pojo.', $setterName, '(_', @name, ');&#10;')" />
                        </xsl:with-param>
                        <xsl:with-param name="resolveRelations" select="'true'" />
                    </xsl:call-template>
                    
                </xsl:when>
            </xsl:choose>
        </xsl:for-each>
    </xsl:template>
    
        
    <xsl:template name="searchSelectClauseProperties">
        <xsl:for-each select="id | property">
            &propertyScopeVars;
            <xsl:choose>
                <xsl:when test="meta/@attribute[local-name() = 'md4j:onsearch-ignore']">
                    <!-- IGNORE  -->
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="concat(',', @name)"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:for-each>
        <xsl:value-of select="meta/@attribute[local-name() = 'md4j:search-select-clause']"/>
    </xsl:template>
    
    <!-- TODO: add action to use the appropriate onFOO-ignore -->
    <xsl:template name="generateProjections">
        <xsl:param name="variableName" select="'projectionProps'" />
        <xsl:param name="action" select="'View'" />
        <xsl:param name="spaceIndent" select="'                '" />
        <xsl:variable name="actionLowcase">
            <xsl:call-template name="string.toLowCase">
                <xsl:with-param name="string" select="$action" />
            </xsl:call-template>
        </xsl:variable>
        <xsl:if test="contains(local-name(), 'subclass')">
            <xsl:variable name="superclass">
                <xsl:call-template name="hbm2all:name.toUnqualifiedClassname">
                    <xsl:with-param name="name" select="@extends" />
                </xsl:call-template>
            </xsl:variable>
            <xsl:for-each select="document(concat($superclass, '.hbm.xml'), .)/hibernate-mapping/class[@name = $superclass]">
                <xsl:call-template name="generateProjections">
                    <xsl:with-param name="variableName" select="$variableName" />
                    <xsl:with-param name="action" select="$action" />
                    <xsl:with-param name="spaceIndent" select="$spaceIndent" />
                </xsl:call-template>
            </xsl:for-each>
        </xsl:if>
        <xsl:for-each select="id | property[not(@formula and $actionType != 'View')]">
            <xsl:if test="not(string(meta[@attribute = concat('md4j:on', $actionLowcase, '-ignore-for-projections')][1]) = 'true')">
                <!-- check if the mapping metadata does not want the property to be rendered -->
                <xsl:value-of select="concat($spaceIndent, $variableName, '.add(&quot;', @name, '&quot;);&#10;')"/>
            </xsl:if>
        </xsl:for-each>
        <xsl:for-each select="many-to-one">
            <xsl:if test="not(string(meta[@attribute = concat('md4j:on', $actionLowcase, '-ignore-for-projections')][1]) = 'true')">
                <xsl:variable name="parent-id-name">
                    <xsl:call-template name="hbm2all:get.superclass.identifier">
                        <xsl:with-param name="superclass"  select="@class"/>
                    </xsl:call-template>
                </xsl:variable>
                <xsl:choose>
                    <xsl:when test="meta[@attribute = 'md4j:project-property' or @attribute = 'md4j:project-property-as-key']">
                        <xsl:variable name="projection-prop" select="meta[@attribute = 'md4j:project-property' or @attribute = 'md4j:project-property-as-key'][1]"/>
                        <xsl:value-of select="concat($spaceIndent, $variableName, '.add(&quot;', @name, '.', $projection-prop, '&quot;);&#10;')"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="concat($spaceIndent, $variableName, '.add(&quot;', @name, '.', $parent-id-name, '&quot;);&#10;')"/>                        
                    </xsl:otherwise>
                </xsl:choose>
                
            </xsl:if>
        </xsl:for-each>
    </xsl:template>
</xsl:transform>